/*
 *	Title:		o3find
 *	Version:	0.62
 *	Created by:	Paolo Copello
 *	Home page:	http://web.tiscali.it/fanelia
 *	
 *	Copyright (C) 2003 Paolo Copello
 *	
 *	******************************************
 *	This file is part of o3find
 *	******************************************
 *	
 *	o3find is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *	
 */



#if !defined(FANEL_FNUTILS_427FDBA3_6CD5_4208_B99A_1570DFDA25B8)
#define FANEL_FNUTILS_427FDBA3_6CD5_4208_B99A_1570DFDA25B8


// define FANEL_FNUTILS_USE_OBSOLETE_LIBRARY when compiling with 
// a library that does NOT support sstream header and 
// istringstream/ostringstream classes

#ifdef FANEL_FNUTILS_USE_OBSOLETE_LIBRARY
#include <strstream>
#else
#include <sstream>
#endif

#include <string>
#include <typeinfo>
#include <stdexcept>
#include <list>
#include <vector>

namespace Fanel 
{

/////////////////////////////////////
// input/output  
/////////////////////////////////////

void WaitForEnter();
void WaitForEnter(std::istream& input);
void ReadLine(std::istream& input, std::string& str);
void ClearInput(std::istream& is);

/////////////////////////////////////
// string operations
/////////////////////////////////////

namespace Str
{
	void ToUpper(std::string& str);
	void ToUpper(const std::string& str, std::string& strOut);
	void Tokenize(const char* str, std::list<std::string>& lst);
	void Tokenize(const std::string& str, std::list<std::string>& lst);
	void Tokenize(const char* str, std::vector<std::string>& lst);
	void Tokenize(const std::string& str, std::vector<std::string>& lst);
	void Epurate(const std::string& in, std::string& out);

	std::string TrimRight( const std::string& s );
	std::string TrimLeft( const std::string& s );
	std::string Trim( const std::string& s );

} // end namespace Str


/////////////////////////////////////
// template conversions 
/////////////////////////////////////

class BadConversion : public std::runtime_error 
{
public:
	BadConversion(const std::string& s)
		: std::runtime_error(s)
	{ }
};

template <class T>
inline std::string ConvertToString(const T& obj)
{
#ifdef FANEL_FNUTILS_USE_OBSOLETE_LIBRARY
	std::ostrstream o; 
#else
	std::ostringstream o;
#endif
	if (!(o << obj))
	{
		throw BadConversion(std::string("ConvertToString(") + typeid(obj).name() + ")");
	}
	std::string strRetVal = o.str();
#ifdef FANEL_FNUTILS_USE_OBSOLETE_LIBRARY
	o.freeze(0);
#endif
	return strRetVal;
}

template <class T>
inline void Convert(const std::string& str, T& target, bool failIfLeftoverChars = true)
{
#ifdef FANEL_FNUTILS_USE_OBSOLETE_LIBRARY
	std::istrstream i(str);
#else
	std::istringstream i(str);
#endif

	char c;
	if (!(i >> target) || (failIfLeftoverChars && i.get(c)))
	{
		throw BadConversion( std::string("Convert(") + str + ", " + typeid(target).name() + ")");
	}
}



} // end namespace Fanel


#endif

